/*CopyrightNotice001

*****************************************************************
                          COPYRIGHT NOTIFICATION
*****************************************************************

THE FOLLOWING IS A NOTICE OF COPYRIGHT, AVAILABILITY OF THE CODE,
AND DISCLAIMER WHICH MUST BE INCLUDED IN THE PROLOGUE OF THE CODE
AND IN ALL SOURCE LISTINGS OF THE CODE.
 
(C)  COPYRIGHT 1995 UNIVERSITY OF CHICAGO
 
Argonne National Laboratory (ANL), with facilities in the States of 
Illinois and Idaho, is owned by the United States Government, and
operated by the University of Chicago under provision of a contract
with the Department of Energy.

Portions of this material resulted from work developed under a U.S.
Government contract and are subject to the following license:  For
a period of five years from June 30, 1995, the Government is
granted for itself and others acting on its behalf a paid-up,
nonexclusive, irrevocable worldwide license in this computer
software to reproduce, prepare derivative works, and perform
publicly and display publicly.  With the approval of DOE, this
period may be renewed for two additional five year periods. 
Following the expiration of this period or periods, the Government
is granted for itself and others acting on its behalf, a paid-up,
nonexclusive, irrevocable worldwide license in this computer
software to reproduce, prepare derivative works, distribute copies
to the public, perform publicly and display publicly, and to permit
others to do so.

*****************************************************************
                                DISCLAIMER
*****************************************************************

NEITHER THE UNITED STATES GOVERNMENT NOR ANY AGENCY THEREOF, NOR
THE UNIVERSITY OF CHICAGO, NOR ANY OF THEIR EMPLOYEES OR OFFICERS,
MAKES ANY WARRANTY, EXPRESS OR IMPLIED, OR ASSUMES ANY LEGAL
LIABILITY OR RESPONSIBILITY FOR THE ACCURACY, COMPLETENESS, OR
USEFULNESS OF ANY INFORMATION, APPARATUS, PRODUCT, OR PROCESS
DISCLOSED, OR REPRESENTS THAT ITS USE WOULD NOT INFRINGE PRIVATELY
OWNED RIGHTS.  

*****************************************************************
LICENSING INQUIRIES MAY BE DIRECTED TO THE INDUSTRIAL TECHNOLOGY
DEVELOPMENT CENTER AT ARGONNE NATIONAL LABORATORY.

CopyrightNotice001*/

/* file: SDDS.h
 * purpose: define structures and prototypes for use with SDDS
 *          format
 *
 * Michael Borland, 1993
 $Log: SDDS.h,v $
 * Revision 1.5  1995/11/13  16:08:13  borland
 * Added first_row_in_mem and writing_page entries to SDDS_DATASET structure
 * to support partially flushed tabular data.
 *
 * Revision 1.4  1995/11/08  04:25:53  borland
 * Added prototypes for new routines FreeArray, DefineParameterLikeArray,
 * and DefineColumnLikeArray.
 *
 * Revision 1.3  1995/09/12  03:18:41  borland
 * Added flag bit #define's for name validity control.
 * Added SDDS_CI_* #defines for SDDS_SetRowsOfInterest case-insensitive
 *   mode.
 * Added SDDS_NOCASE_COMPARE for SDDS_MatchRowsOfInterest case-insenstive
 *   mode.
 *
 * Revision 1.2  1995/09/05  21:14:46  saunders
 * First test release of the SDDS1.5 package.
 *
 */

#if !defined(_SDDS_)
#include <stdio.h>
#include <stdarg.h>

#ifdef __cplusplus 
extern "C" {
#endif


#define _SDDS_ 1

#define SDDS_VERSION 1

#include "SDDStypes.h"

#define RW_ASSSOCIATES 0

#define SDDS_BINARY 1
#define SDDS_ASCII  2
#define SDDS_NUM_DATA_MODES 2

extern char *SDDS_type_name[SDDS_NUM_TYPES];
extern long SDDS_type_size[SDDS_NUM_TYPES];
extern char *SDDS_data_mode[SDDS_NUM_DATA_MODES];

/* this shouldn't be changed without changing buffer sizes in namelist routines: */
#define SDDS_MAXLINE 1024

#define SDDS_NORMAL_DEFINITION 0
#define SDDS_WRITEONLY_DEFINITION 1

typedef struct {
    char *name, *symbol, *units, *description, *format_string;
    long type;
    char *fixed_value;
    /* these are used internally and are not set by the user: */
    long definition_mode, memory_number;
    } PARAMETER_DEFINITION;
#define SDDS_PARAMETER_FIELDS 7

typedef struct {
    char *name, *symbol, *units, *description, *format_string;
    long type, field_length;
    /* these are used internally and are not set by the user: */
    long definition_mode, memory_number, pointer_number;
    } COLUMN_DEFINITION;
#define SDDS_COLUMN_FIELDS 7

typedef struct {
    char *name, *symbol, *units, *description, *format_string, *group_name;
    long type, field_length, dimensions;
    } ARRAY_DEFINITION;
#define SDDS_ARRAY_FIELDS 9

typedef struct {
    char *name, *filename, *path, *description, *contents;
    long sdds;
    } ASSOCIATE_DEFINITION;
#define SDDS_ASSOCIATE_FIELDS 6

typedef struct {
    long mode, lines_per_row, no_row_counts;
    long additional_header_lines;
    } DATA_MODE;
typedef struct {
    long n_columns, n_parameters, n_associates, n_arrays;

    char *description;
    char *contents;
    long version;
    DATA_MODE data_mode;
    COLUMN_DEFINITION *column_definition;
    PARAMETER_DEFINITION *parameter_definition;
    ARRAY_DEFINITION *array_definition;
    ASSOCIATE_DEFINITION *associate_definition;

    char *filename;
    FILE *fp;
    } SDDS_LAYOUT;

typedef struct {
    ARRAY_DEFINITION *definition;
    long *dimension, elements;
    /* Address of array of data values, stored contiguously.
     * For STRING data, the "data values" are actually the addresses of the individual strings.
     */
    void *data;
    void *pointer;
    } SDDS_ARRAY;

typedef struct {
    SDDS_LAYOUT layout, original_layout;
    long page_number;

    long rowcount_offset; /* ftell() position of row count */
    long n_rows_written;  /* number of tabular data rows written to disk */
    long last_row_written; /* virtual index of last row written */
    long first_row_in_mem; /* virtual index of first row in memory */
    short writing_page;    /* 1/0 for writing/not-writing page */
    long n_rows_allocated; /* number of tabular data rows for which space is alloc'd */
    long n_rows;           /* number of rows stored in memory */
    long *row_flag;        /* accept/reject flags for rows stored in memory */

    long n_of_interest;
    long *column_order;          /* column_order[i] = internal index of user's ith column */
    long *column_flag;           /* column_flag[i] indicates whether internal ith column has been selected */

    /* array of SDDS_ARRAY structures for storing array data */
    SDDS_ARRAY *array;

    /* array for parameter data.  The address of the data for the ith parameter
     * is parameter[i].  So *(<type-name> *)parameter[i] gives the data itself.  For type
     * SDDS_STRING the "data itself" is actually the address of the string, and the type-name
     * is "char *".
     */
    void **parameter;

    /* array for tabular data.  The starting address for the data for the ith column
     * is data[i].  So ((<type-name> *)data[i])[j] gives the jth data value.  
     * For type SDDS_STRING the "data value" is actually the address of the string, and
     * the type-name is "char *".
     */
    void **data;
    } SDDS_DATASET;

typedef SDDS_DATASET SDDS_TABLE;

/* prototypes for routines to prepare and write SDDS files */
extern long SDDS_InitializeAppend(SDDS_DATASET *SDDS_dataset, char *filename);
extern long SDDS_InitializeOutput(SDDS_DATASET *SDDS_dataset, long data_mode, long lines_per_row, char *description,
                           char *contents, char *filename);
extern long SDDS_Terminate(SDDS_DATASET *SDDS_dataset);
extern void SDDS_SetTerminateMode(unsigned long mode);
#define TERMINATE_DONT_FREE_TABLE_STRINGS 0x0001
#define TERMINATE_DONT_FREE_ARRAY_STRINGS 0x0002
extern long SDDS_DefineParameter(SDDS_DATASET *SDDS_dataset, char *name, char *symbol, char *units, char *description, 
                           char *format_string, long type, char *fixed_value);
extern long SDDS_DefineParameter1(SDDS_DATASET *SDDS_dataset, char *name, char *symbol, char *units, char *description, 
                           char *format_string, long type, void *fixed_value);
extern long SDDS_DefineColumn(SDDS_DATASET *SDDS_dataset, char *name, char *symbol, char *units, char *description, 
                        char *format_string, long type, long field_length);
extern long SDDS_DefineArray(SDDS_DATASET *SDDS_dataset, char *name, char *symbol, char *units, char *description, 
                        char *format_string, long type, long field_length, long dimensions, char *group_name);
extern long SDDS_DefineAssociate(SDDS_DATASET *SDDS_dataset, char *name,
                                 char *filename, char *path, char *description, char *contents, long sdds);
extern long SDDS_IsValidName(char *name, char *dataClass);
extern long SDDS_SetNameValidityFlags(unsigned long flags);
#define SDDS_ALLOW_ANY_NAME 0x0001UL
#define SDDS_ALLOW_V15_NAME 0x0002UL
extern long SDDS_DefineSimpleColumn(SDDS_DATASET *SDDS_dataset, char *name, char *unit, long type);
extern long SDDS_DefineSimpleParameter(SDDS_DATASET *SDDS_dataset, char *name, char *unit, long type);
extern long SDDS_DefineSimpleColumns(SDDS_DATASET *SDDS_dataset, long number, char **name, char **unit, long type);
extern long SDDS_DefineSimpleParameters(SDDS_DATASET *SDDS_dataset, long number, char **name, char **unit, long type);
extern long SDDS_WriteLayout(SDDS_DATASET *SDDS_dataset);
extern long SDDS_EraseData(SDDS_DATASET *SDDS_dataset);

extern long SDDS_ProcessColumnString(SDDS_DATASET *SDDS_dataset, char *string, long mode);
extern long SDDS_ProcessParameterString(SDDS_DATASET *SDDS_dataset, char *string, long mode);
extern long SDDS_ProcessArrayString(SDDS_DATASET *SDDS_dataset, char *string);
extern long SDDS_ProcessAssociateString(SDDS_DATASET *SDDS_dataset, char *string);

extern long SDDS_InitializeCopy(SDDS_DATASET *SDDS_target, SDDS_DATASET *SDDS_source, char *filename, char *filemode);
extern long SDDS_CopyLayout(SDDS_DATASET *SDDS_target, SDDS_DATASET *SDDS_source);
extern long SDDS_CopyPage(SDDS_DATASET *SDDS_target, SDDS_DATASET *SDDS_source);
#define SDDS_CopyTable(a, b) SDDS_CopyPage(a, b)
extern long SDDS_CopyParameters(SDDS_DATASET *SDDS_target, SDDS_DATASET *SDDS_source);
extern long SDDS_CopyArrays(SDDS_DATASET *SDDS_target, SDDS_DATASET *SDDS_source);
extern long SDDS_CopyColumns(SDDS_DATASET *SDDS_target, SDDS_DATASET *SDDS_source);
extern long SDDS_CopyRowsOfInterest(SDDS_DATASET *SDDS_target, SDDS_DATASET *SDDS_source);
extern long SDDS_CopyRow(SDDS_DATASET *SDDS_target, long target_row, SDDS_DATASET *SDDS_source, long source_srow);
extern long SDDS_CopyRowDirect(SDDS_DATASET *SDDS_target, long target_row, SDDS_DATASET *SDDS_source, long source_row);
extern long SDDS_CopyAdditionalRows(SDDS_DATASET *SDDS_target, SDDS_DATASET *SDDS_source);

extern long SDDS_DeferSavingLayout(long mode);
extern long SDDS_SaveLayout(SDDS_DATASET *SDDS_dataset);
extern long SDDS_RestoreLayout(SDDS_DATASET *SDDS_dataset);

#define SDDS_BY_INDEX 1
#define SDDS_BY_NAME  2

extern long SDDS_StartPage(SDDS_DATASET *SDDS_dataset, long expected_n_rows);
#define SDDS_StartTable(a, b) SDDS_StartPage(a, b)
extern long SDDS_LengthenTable(SDDS_DATASET *SDDS_dataset, long n_additional_rows);
#define SDDS_SET_BY_INDEX SDDS_BY_INDEX
#define SDDS_SET_BY_NAME SDDS_BY_NAME
#define SDDS_PASS_BY_VALUE 4
#define SDDS_PASS_BY_REFERENCE 8
#define SDDS_PASS_BY_STRING 16
extern long SDDS_SetParameters(SDDS_DATASET *SDDS_dataset, long mode, ...);
extern long SDDS_SetRowValues(SDDS_DATASET *SDDS_dataset, long mode, long row, ...);
extern long SDDS_WritePage(SDDS_DATASET *SDDS_dataset);
#define SDDS_WriteTable(a) SDDS_WritePage(a)
extern long SDDS_UpdatePage(SDDS_DATASET *SDDS_dataset, unsigned long mode);
#define FLUSH_TABLE 0x1UL
#define SDDS_UpdateTable(a) SDDS_UpdatePage(a, 0)
extern long SDDS_SetColumn(SDDS_DATASET *SDDS_dataset, long mode, void *data, long rows, ...);
extern long SDDS_SetColumnFromDoubles(SDDS_DATASET *SDDS_dataset, long mode, double *data, long rows, ...);
extern long SDDS_SetColumnFromLongs(SDDS_DATASET *SDDS_dataset, long mode, long *data, long rows, ...);
extern long SDDS_SetParametersFromDoubles(SDDS_DATASET *SDDS_dataset, long mode, ...);

#define SDDS_GET_BY_INDEX SDDS_BY_INDEX
#define SDDS_GET_BY_NAME SDDS_BY_NAME
extern long SDDS_GetColumnInformation(SDDS_DATASET *SDDS_dataset, char *field_name, void *memory, long mode, ...);
extern long SDDS_GetParameterInformation(SDDS_DATASET *SDDS_dataset, char *field_name, void *memory, long mode, ...);
extern long SDDS_GetArrayInformation(SDDS_DATASET *SDDS_dataset, char *field_name, void *memory, long mode, ...);
extern long SDDS_GetAssociateInformation(SDDS_DATASET *SDDS_dataset, char *field_name, void *memory, long mode, ...);
extern long SDDS_ChangeColumnInformation(SDDS_DATASET *SDDS_dataset, char *field_name, void *memory, long mode, ...);
extern long SDDS_ChangeParameterInformation(SDDS_DATASET *SDDS_dataset, char *field_name, void *memory, long mode, ...);
extern long SDDS_ChangeArrayInformation(SDDS_DATASET *SDDS_dataset, char *field_name, void *memory, long mode, ...);

/* prototypes for routines to read and use SDDS files  */
extern long SDDS_InitializeInput(SDDS_DATASET *SDDS_dataset, char *filename);
extern long SDDS_InitializeHeaderlessInput(SDDS_DATASET *SDDS_dataset, char *filename);
extern long SDDS_ReadPage(SDDS_DATASET *SDDS_dataset);
#define SDDS_ReadTable(a) SDDS_ReadPage(a)
extern long SDDS_ReadRecoveryPossible(void);

extern long SDDS_SetColumnFlags(SDDS_DATASET *SDDS_dataset, long column_flag_value);
extern long SDDS_SetRowFlags(SDDS_DATASET *SDDS_dataset, long row_flag_value);
extern long SDDS_GetRowFlag(SDDS_DATASET *SDDS_dataset, long row);
extern long SDDS_GetRowFlags(SDDS_DATASET *SDDS_dataset, long *flag, long rows);
/* logic flags for SDDS_AssertRowFlags and SDDS_AssertColumnFlags */
#define SDDS_FLAG_ARRAY  0x001UL
#define SDDS_INDEX_LIMITS 0x002UL
extern long SDDS_AssertRowFlags(SDDS_DATASET *SDDS_dataset, unsigned long mode, ...);
/* modes for SDDS_SetColumnsOfInterest and SDDS_SetRowsOfInterest: */
#define SDDS_NAME_ARRAY 1
#define SDDS_NAMES_STRING 2
#define SDDS_NAME_STRINGS 3
#define SDDS_MATCH_STRING 4
#define SDDS_CI_NAME_ARRAY 5
#define SDDS_CI_NAMES_STRING 6
#define SDDS_CI_NAME_STRINGS 7
#define SDDS_CI_MATCH_STRING 8

/* logic flags for SDDS_SetColumnsOfInterest, SDDS_SetRowsOfInterest, and SDDS_MatchRowsOfInterest: */
#define SDDS_AND               0x0001UL
#define SDDS_OR                0x0002UL
#define SDDS_NEGATE_MATCH      0x0004UL
#define SDDS_NEGATE_PREVIOUS   0x0008UL
#define SDDS_NEGATE_EXPRESSION 0x0010UL
#define SDDS_INDIRECT_MATCH    0x0020UL
#define SDDS_1_PREVIOUS        0x0040UL
#define SDDS_0_PREVIOUS        0x0080UL
/* used by MatchRowsOfInterest only at this point: */
#define SDDS_NOCASE_COMPARE    0x0100UL

extern long SDDS_MatchColumns(SDDS_DATASET *SDDS_dataset, char ***match, long matchMode, long typeMode, ... );
extern long SDDS_MatchParameters(SDDS_DATASET *SDDS_dataset, char ***match, long matchMode, long typeMode, ... );
extern long SDDS_MatchArrays(SDDS_DATASET *SDDS_dataset, char ***match, long matchMode, long typeMode, ... );
extern long SDDS_Logic(long previous, long match, unsigned long logic);
extern long SDDS_SetColumnsOfInterest(SDDS_DATASET *SDDS_dataset, long mode, ...);
extern long SDDS_AssertColumnFlags(SDDS_DATASET *SDDS_dataset, unsigned long mode, ...);
extern long SDDS_SetRowsOfInterest(SDDS_DATASET *SDDS_dataset, char *selection_column, long mode, ...);
extern long SDDS_MatchRowsOfInterest(SDDS_DATASET *SDDS_dataset, char *selection_column, char *label_to_match, long logic);
extern long SDDS_DeleteColumn(SDDS_DATASET *SDDS_dataset, char *column_name);
extern long SDDS_DeleteParameter(SDDS_DATASET *SDDS_dataset, char *parameter_name);
extern long SDDS_DeleteUnsetColumns(SDDS_DATASET *SDDS_dataset);
extern long SDDS_CountColumnsOfInterest(SDDS_DATASET *SDDS_dataset);
extern long SDDS_ColumnIsOfInterest(SDDS_DATASET *SDDS_dataset, char *name);
extern long SDDS_ColumnCount(SDDS_DATASET *dataset);
extern long SDDS_ParameterCount(SDDS_DATASET *dataset);
extern long SDDS_ArrayCount(SDDS_DATASET *dataset);
extern long SDDS_CountRowsOfInterest(SDDS_DATASET *SDDS_dataset);
extern long SDDS_DeleteUnsetRows(SDDS_DATASET *SDDS_dataset);
extern long SDDS_FilterRowsOfInterest(SDDS_DATASET *SDDS_dataset, char *filter_column, double lower, double upper, long logic);
extern long SDDS_ItemInsideWindow(void *data, long index, long type, double lower_limit, double upper_limit);

extern void *SDDS_GetColumn(SDDS_DATASET *SDDS_dataset, char *column_name);
extern double *SDDS_GetColumnInDoubles(SDDS_DATASET *SDDS_dataset, char *column_name);
extern void *SDDS_GetNumericColumn(SDDS_DATASET *SDDS_dataset, char *column_name, long desiredType);
extern void *SDDS_GetRow(SDDS_DATASET *SDDS_dataset, long srow_index, void *memory);
extern void *SDDS_GetValue(SDDS_DATASET *SDDS_dataset, char *column_name, long srow_index, void *memory);
extern void *SDDS_GetValueByIndex(SDDS_DATASET *SDDS_dataset, long column_index, long srow_index, void *memory);
extern void *SDDS_GetValueByAbsIndex(SDDS_DATASET *SDDS_dataset, long column_index, long srow_index, void *memory);
extern void *SDDS_GetParameter(SDDS_DATASET *SDDS_dataset, char *parameter_name, void *memory);
extern double *SDDS_GetParameterAsDouble(SDDS_DATASET *SDDS_dataset, char *parameter_name, double *data);
extern long SDDS_GetParameters(SDDS_DATASET *SDDS_dataset, ...);
extern void *SDDS_GetFixedValueParameter(SDDS_DATASET *SDDS_dataset, char *parameter_name, void *memory);
extern long SDDS_GetDescription(SDDS_DATASET *SDDS_dataset, char **text, char **contents);

extern void *SDDS_GetMatrixOfRows(SDDS_DATASET *SDDS_dataset, long *n_rows);
extern void *SDDS_GetCastMatrixOfRows(SDDS_DATASET *SDDS_dataset, long *n_rows, long sddsType);
#define SDDS_ROW_MAJOR_DATA 1
#define SDDS_COLUMN_MAJOR_DATA 2
extern void *SDDS_GetMatrixFromColumn(SDDS_DATASET *SDDS_dataset, char *column_name, long dimension1, long dimension2, long mode);
extern void *SDDS_GetDoubleMatrixFromColumn(SDDS_DATASET *SDDS_dataset, char *column_name, long dimension1, long dimension2, long mode);

extern SDDS_ARRAY *SDDS_GetArray(SDDS_DATASET *SDDS_dataset, char *array_name, SDDS_ARRAY *memory);
#define SDDS_POINTER_ARRAY 1
#define SDDS_CONTIGUOUS_DATA 2
extern long SDDS_SetArrayVararg(SDDS_DATASET *SDDS_dataset, char *array_name, long mode, void *data_pointer, ...);
extern long SDDS_SetArray(SDDS_DATASET *SDDS_dataset, char *array_name, long mode, void *data_pointer, long *dimension);

/* error-handling and utility routines: */
extern void *SDDS_Realloc(void *old_ptr, long new_size);
extern long SDDS_NumberOfErrors();
extern void SDDS_ClearErrors();
extern void SDDS_SetError(char *error_text);
extern void SDDS_Bomb(char *message);
extern long SDDS_RegisterProgramName(char *name);
#define SDDS_VERBOSE_PrintErrors 1
#define SDDS_EXIT_PrintErrors 2
extern void SDDS_PrintErrors(FILE *fp, long mode);

extern char **SDDS_GetColumnNames(SDDS_DATASET *SDDS_dataset, long *number);
extern char **SDDS_GetParameterNames(SDDS_DATASET *SDDS_dataset, long *number);
extern char **SDDS_GetAssociateNames(SDDS_DATASET *SDDS_dataset, long *number);
extern char **SDDS_GetArrayNames(SDDS_DATASET *SDDS_dataset, long *number);

extern COLUMN_DEFINITION *SDDS_GetColumnDefinition(SDDS_DATASET *SDDS_dataset, char *name);
extern COLUMN_DEFINITION *SDDS_CopyColumnDefinition(COLUMN_DEFINITION **target, COLUMN_DEFINITION *source);
extern long SDDS_FreeColumnDefinition(COLUMN_DEFINITION *source);
extern long SDDS_TransferColumnDefinition(SDDS_DATASET *target, SDDS_DATASET *source, char *name, char *newName);
extern long SDDS_DefineColumnLikeParameter(SDDS_DATASET *target, SDDS_DATASET *source, char *name, char *newName);
extern long SDDS_DefineColumnLikeArray(SDDS_DATASET *target, SDDS_DATASET *source, char *name, char *newName);

extern PARAMETER_DEFINITION *SDDS_GetParameterDefinition(SDDS_DATASET *SDDS_dataset, char *name);
extern PARAMETER_DEFINITION *SDDS_CopyParameterDefinition(PARAMETER_DEFINITION **target, PARAMETER_DEFINITION *source);
extern long SDDS_FreeParameterDefinition(PARAMETER_DEFINITION *source);
extern long SDDS_TransferParameterDefinition(SDDS_DATASET *target, SDDS_DATASET *source, char *name, char *newName);
extern long SDDS_DefineParameterLikeColumn(SDDS_DATASET *target, SDDS_DATASET *source, char *name, char *newName);
extern long SDDS_DefineParameterLikeArray(SDDS_DATASET *target, SDDS_DATASET *source, char *name, char *newName);

extern ARRAY_DEFINITION *SDDS_GetArrayDefinition(SDDS_DATASET *SDDS_dataset, char *name);
extern ARRAY_DEFINITION *SDDS_CopyArrayDefinition(ARRAY_DEFINITION **target, ARRAY_DEFINITION *source);
extern long SDDS_FreeArrayDefinition(ARRAY_DEFINITION *source);
extern long SDDS_TransferArrayDefinition(SDDS_DATASET *target, SDDS_DATASET *source, char *name, char *newName);

extern ASSOCIATE_DEFINITION *SDDS_GetAssociateDefinition(SDDS_DATASET *SDDS_dataset, char *name);
extern ASSOCIATE_DEFINITION *SDDS_CopyAssociateDefinition(ASSOCIATE_DEFINITION **target, ASSOCIATE_DEFINITION *source);
extern long SDDS_FreeAssociateDefinition(ASSOCIATE_DEFINITION *source);
extern long SDDS_TransferAssociateDefinition(SDDS_DATASET *target, SDDS_DATASET *source, char *name, char *newName);

extern long SDDS_GetColumnIndex(SDDS_DATASET *SDDS_dataset, char *name);
extern long SDDS_GetParameterIndex(SDDS_DATASET *SDDS_dataset, char *name);
extern long SDDS_GetArrayIndex(SDDS_DATASET *SDDS_dataset, char *name);
extern long SDDS_GetAssociateIndex(SDDS_DATASET *SDDS_dataset, char *name);
extern long SDDS_GetColumnType(SDDS_DATASET *SDDS_dataset, long index);
extern long SDDS_GetNamedColumnType(SDDS_DATASET *SDDS_dataset, char *name);
extern long SDDS_GetParameterType(SDDS_DATASET *SDDS_dataset, long index);
extern long SDDS_GetNamedParameterType(SDDS_DATASET *SDDS_dataset, char *name);
extern long SDDS_GetArrayType(SDDS_DATASET *SDDS_dataset, long index);
extern long SDDS_GetNamedArrayType(SDDS_DATASET *SDDS_dataset, char *name);
extern long SDDS_GetTypeSize(long type);
extern char *SDDS_GetTypeName(long type);
extern long SDDS_IdentifyType(char *typeName);

#define FIND_ANY_TYPE       0
#define FIND_SPECIFIED_TYPE 1
#define FIND_NUMERIC_TYPE   2
#define FIND_INTEGER_TYPE   3
#define FIND_FLOATING_TYPE  4

extern char *SDDS_FindColumn(SDDS_DATASET *SDDS_dataset,  long mode, ...);
extern char *SDDS_FindParameter(SDDS_DATASET *SDDS_dataset,  long mode, ...);
extern char *SDDS_FindArray(SDDS_DATASET *SDDS_dataset,  long mode, ...);

extern long SDDS_CheckColumn(SDDS_DATASET *SDDS_dataset, char *name, char *units, long type, FILE *fp_message);
extern long SDDS_CheckParameter(SDDS_DATASET *SDDS_dataset, char *name, char *units, long type, FILE *fp_message);
extern long SDDS_CheckArray(SDDS_DATASET *SDDS_dataset, char *name, char *units, long type, FILE *fp_message);
extern long SDDS_PrintCheckText(FILE *fp, char *name, char *units, long type, char *class_name, long error_code);
#define SDDS_CHECK_OKAY 0
#define SDDS_CHECK_OK SDDS_CHECK_OKAY
#define SDDS_CHECK_NONEXISTENT 1
#define SDDS_CHECK_WRONGTYPE  2
#define SDDS_CHECK_WRONGUNITS  3

extern long SDDS_IsActive(SDDS_DATASET *SDDS_dataset);

extern long SDDS_CopyString(char **target, char *source);
extern long SDDS_CopyStringArray(char **target, char **source, long n_strings);
extern long SDDS_VerifyPrintfFormat(char *format_string, long type);
extern long SDDS_HasWhitespace(char *string);
extern char *fgetsSkipComments(char *s, long slen, FILE *fp, char skip_char);
extern char *fgetsSkipCommentsResize(char **s, long *slen, FILE *fp, char skip_char);
extern void SDDS_CutOutComments(char *s, char cc);
extern void SDDS_EscapeNewlines(char *s);
extern void SDDS_EscapeQuotes(char *s, char quote_char);
extern void SDDS_UnescapeQuotes(char *s, char quote_char);
extern long SDDS_GetToken(char *s, char *buffer, long buflen);
extern long SDDS_PadToLength(char *string, long length);
extern void SDDS_EscapeCommentCharacters(char *string, char cc);
extern long SDDS_ZeroMemory(void *mem, long n_bytes);
extern long SDDS_SetMemory(void *mem, long n_elements, long data_type, ...);
extern long SDDS_PrintTypedValue(void *data, long index, long type, char *format, FILE *fp);
extern long SDDS_WriteTypedValue(void *data, long index, long type, char *format, FILE *fp);
extern long SDDS_SprintTypedValue(void *data, long index, long type, char *format, char *buffer);
extern void *SDDS_CastValue(void *data, long index, long data_type, long desired_type, void *memory);
extern void SDDS_RemovePadding(char *s);
extern long SDDS_StringIsBlank(char *s);
extern void *SDDS_AllocateMatrix(long size, long dim1, long dim2);
extern void SDDS_FreeMatrix(void **ptr, long dim1);
extern void SDDS_FreeArray(SDDS_ARRAY *array);
extern void *SDDS_MakePointerArray(void *data, long type, long dimensions, long *dimension);
extern long SDDS_ApplyFactorToParameter(SDDS_DATASET *SDDS_dataset, char *name, double factor);
extern long SDDS_ApplyFactorToColumn(SDDS_DATASET *SDDS_dataset, char *name, double factor);
extern long SDDS_DeleteParameterFixedValues(SDDS_DATASET *SDDS_dataset);
extern long SDDS_SetDataMode(SDDS_DATASET *SDDS_dataset, long newmode);
extern long SDDS_CheckDatasetStructureSize(long size);
#define SDDS_CheckTableStructureSize(a) SDDS_CheckDatasetStructureSize(a)

extern long SDDS_ScanData(char *string, long type, long field_length, void *data, long index, long is_parameter);

extern double SDDS_ConvertToDouble(long type, void *data, long index);
extern long SDDS_CreateRpnMemory(char *name);

#if defined(RPN_SUPPORT)
extern long SDDS_FilterRowsWithRpnTest(SDDS_DATASET *SDDS_dataset, char *rpn_test);
extern long SDDS_StoreParametersInRpnMemories(SDDS_DATASET *SDDS_dataset);
extern long SDDS_StoreRowInRpnMemories(SDDS_DATASET *SDDS_dataset, long row);
extern long SDDS_StoreColumnsInRpnArrays(SDDS_DATASET *SDDS_dataset);
extern long SDDS_ComputeColumn(SDDS_DATASET *SDDS_dataset, long column, char *equation);
extern long SDDS_ComputeParameter(SDDS_DATASET *SDDS_dataset, long column, char *equation);
#endif


#ifdef __cplusplus
}
#endif

#endif
