#ifndef INC_arCS_h
#define INC_arCS_h
/*	$Id: arCS.h,v 1.1 1995/02/13 15:33:16 jba Exp $
 *	Author:	Roger A. Cole
 *	Date:	03-30-90
 *
 *	Experimental Physics and Industrial Control System (EPICS)
 *
 *	Copyright 1991, the Regents of the University of California,
 *	and the University of Chicago Board of Governors.
 *
 *	This software was produced under  U.S. Government contracts:
 *	(W-7405-ENG-36) at the Los Alamos National Laboratory,
 *	and (W-31-109-ENG-38) at Argonne National Laboratory.
 *
 *	Initial development by:
 *		The Controls and Automation Group (AT-8)
 *		Ground Test Accelerator
 *		Accelerator Technology Division
 *		Los Alamos National Laboratory
 *
 *	Co-developed with
 *		The Controls and Computing Group
 *		Accelerator Systems Division
 *		Advanced Photon Source
 *		Argonne National Laboratory
 *
 * Modification Log:
 * -----------------
 * .00	03-30-90	rac	initial version
 * .01	06-19-91	rac	installed in SCCS
 *
 */
/*+/mod***********************************************************************
* TITLE	arCS.h - control system related definitions
*
* DESCRIPTION
*	This file contains various definitions related to control system
*	characteristics, primarily as needed by the AR code.
*
*	Many of these definitions are used in the layout of structures in
*	AR data files, so arbitrarily changing size-related definitions
*	would probably require throwing away old files and starting with
*	fresh ones.
*
*	It is recommended to call arCSCheck() as an 'initialization' step
*	when using definitions from this file.  arCSCheck() compares these
*	definitions with actual GTACS definitions and aborts if differences
*	are detected.
*
* BUGS
* o	most of the definitions here aren't coupled directly to definitions
*	in the various include files used to implement control system
*	software.  Thus, the definitions here don't stay in sync with the
*	"real" definitions.
*
* SEE ALSO:
* o	arCSCheck.c	checks for validity of definitions
* o	db_access.h	contains definitions for database related
*			quantities.
* o	cadef.h		contains some additional macro and symbol definitions
*
*-***************************************************************************/


/*----------------------------------------------------------------------------
* STRINGS and other data items
*
*    strings are always null terminated, so that the allowable length is
*    AR_xxx_DIM - 1
*
*---------------------------------------------------------------------------*/
#define AR_STRVAL_DIM 40	/* dimension for string values.
				For an array of string values, each string
				must have this dimension--other dimensions
				WILL NOT WORK!! */
#define AR_NAME_DIM 36		/* dimension for names, including channel
				names, request names, etc.  The most critical
				issue is channel names--as of this writing,
				record names are 28 char and field names are
				4 char.  (Channel name is recordName.fldName) */
#define AR_UNITS_DIM 8		/* dimension for units text */
#define AR_DESC_DIM 24		/* dimension for record description */
#define AR_STATE_DIM 16		/* number of possible states */
#define AR_STATE_TEXT_DIM 26	/* dimension for state strings */
		/* usage: char stateTable[AR_STATE_DIM][AR_STATE_TEXT_DIM] */

typedef unsigned char AR_ALM_SEV, AR_ALM_STAT;


#endif
