/*CopyrightNotice001

*****************************************************************
                          COPYRIGHT NOTIFICATION
*****************************************************************

THE FOLLOWING IS A NOTICE OF COPYRIGHT, AVAILABILITY OF THE CODE,
AND DISCLAIMER WHICH MUST BE INCLUDED IN THE PROLOGUE OF THE CODE
AND IN ALL SOURCE LISTINGS OF THE CODE.
 
(C)  COPYRIGHT 1995 UNIVERSITY OF CHICAGO
 
Argonne National Laboratory (ANL), with facilities in the States of 
Illinois and Idaho, is owned by the United States Government, and
operated by the University of Chicago under provision of a contract
with the Department of Energy.

Portions of this material resulted from work developed under a U.S.
Government contract and are subject to the following license:  For
a period of five years from June 30, 1995, the Government is
granted for itself and others acting on its behalf a paid-up,
nonexclusive, irrevocable worldwide license in this computer
software to reproduce, prepare derivative works, and perform
publicly and display publicly.  With the approval of DOE, this
period may be renewed for two additional five year periods. 
Following the expiration of this period or periods, the Government
is granted for itself and others acting on its behalf, a paid-up,
nonexclusive, irrevocable worldwide license in this computer
software to reproduce, prepare derivative works, distribute copies
to the public, perform publicly and display publicly, and to permit
others to do so.

*****************************************************************
                                DISCLAIMER
*****************************************************************

NEITHER THE UNITED STATES GOVERNMENT NOR ANY AGENCY THEREOF, NOR
THE UNIVERSITY OF CHICAGO, NOR ANY OF THEIR EMPLOYEES OR OFFICERS,
MAKES ANY WARRANTY, EXPRESS OR IMPLIED, OR ASSUMES ANY LEGAL
LIABILITY OR RESPONSIBILITY FOR THE ACCURACY, COMPLETENESS, OR
USEFULNESS OF ANY INFORMATION, APPARATUS, PRODUCT, OR PROCESS
DISCLOSED, OR REPRESENTS THAT ITS USE WOULD NOT INFRINGE PRIVATELY
OWNED RIGHTS.  

*****************************************************************
LICENSING INQUIRIES MAY BE DIRECTED TO THE INDUSTRIAL TECHNOLOGY
DEVELOPMENT CENTER AT ARGONNE NATIONAL LABORATORY.

CopyrightNotice001*/
/* file   : mdb.h
 * purpose: definitions for general use, for mdblib, and for mdbmth.
 *
 * Michael Borland, 1988
 $Log: mdb.h,v $
 * Revision 1.4  1995/11/13  16:08:35  borland
 * Added prototype for function mtime().
 *
 * Revision 1.3  1995/09/12  03:19:44  borland
 * Added prototypes for wild_match_ci, strchr_ci, strcmp_ci
 *
 * Revision 1.2  1995/09/05  21:15:14  saunders
 * First test release of the SDDS1.5 package.
 *
 */
#ifndef _MDB_
#define _MDB_ 1

#if defined(SUNOS4)
#define SUN_SPARC 1
#endif

#include <string.h>
#include <stdio.h>

#if defined(SUNOS4) && defined(GNU_C)
/* prototypes for functions not defined in stdio: */
extern int printf(const char *format_spec, ...);
extern int fprintf(FILE *file_ptr, const char *format_spec, ...);
/* int sprintf(char *str, const char *format_spec, ...); */
extern int scanf(const char *format_spec, ...);
extern int fscanf(FILE *file_ptr, const char *format_spec, ...);
extern int sscanf(char *str, const char *format_spec, ...);
extern int fputs(const char *string, FILE *file_ptr);
extern int puts(const char *string);
extern int fputc(char c, FILE *file_ptr);
extern int fclose(FILE *file_ptr);
extern int close(int file_descriptor);
extern void perror(char *s);
extern int fseek(FILE *file_ptr, long offset, int direction);
extern int fread(void *data, int size, int number, FILE *fp);
extern int fwrite(void *data, int size, int number, FILE *fp);
extern int fflush(FILE *file_ptr);
/* prototypes for functions not fully prototyped in math.h: */
extern double   acos(double x);
extern double   asin(double x);
extern double   atan(double x);
extern double   atan2(double y, double x);
extern double   ceil(double x);
extern double   cos(double x);
extern double   cosh(double x);
extern double   exp(double x);
extern double   fabs(double x);
extern double   floor(double x);
extern double   fmod(double x, double y);
extern double   frexp(double value, int *expo);
extern double   ldexp(double value, int expo);
extern double   log(double x);
extern double   log10(double x);
extern double   modf(double value, double *iptr);
extern double   pow(double x, double y);
extern double   sin(double x);
extern double   sinh(double x);
extern double   sqrt(double x);
extern double   tan(double x);
extern double   tanh(double x);
#endif

#include <stdlib.h>

void *array_1d(long size_of_elem, long lower_index, long upper_index);
void **array_2d(long size_of_elem, long lower1, long upper1, long lower2,
                long upper2);
int free_array_1d(void *array, long size_of_elem, long lower_index,
                long upper_index);
int free_array_2d(void **array, long size_of_elem, long lower1, long upper1,
                long lower2, long upper2);
void **zarray_2d(long size, long n1, long n2);
void **resize_zarray_2d(long size, long old_n1, long old_n2,
            void **array, long n1, long n2);
int free_zarray_2d(void **array, long n1, long n2);
void zero_memory(void *memory, long n_bytes);
int tfree(void *ptr);

void fill_int_array(int *array, long n, int value);
void fill_short_array(short *array, long n, short value);
void fill_long_array(long *array, long n, long value);
void fill_float_array(float *array, long n, float value);
void fill_double_array(double *array, long n, double value);

void *tmalloc(unsigned long size_of_block);
void *trealloc(void *ptr, unsigned long size_of_block);

/* String-related macro definitions: */
#define query(s, t) ((*(t)=0),fputs(s,stdout),gets(t))
#define query_e(s, t) ((*(t)=0),fputs(s,stderr),gets(t))

#define is_yes(c) ((c)=='y' || (c)=='Y')
#define is_no(c) ((c)=='n' || (c)=='N')

#define chop_nl(m_s) ( ((m_s)[strlen(m_s)-1]=='\n') ? (m_s)[strlen(m_s)-1]=0 : 0)


/*   -- Data-scanning routines: */
extern long   query_long(char *prompt, long default_value);
extern int    query_int(char *prompt, int default_value);
extern short  query_short(char *prompt, short default_value);
extern double query_double(char *prompt, double default_value);
extern float  query_float(char *prompt, float default_value);
extern int   get_double(double *target, char *source);
extern int   get_long(long *target, char *source);
extern int   get_short(short *target, char *source);
extern int   get_int(int *target, char *source);
extern int   get_float(float *target, char *source);
extern char  *get_token(char *source);
extern char  *get_token_buf(char *source, char *buffer, long buffer_length);
extern char  *get_token_t(char *source, char *token_delimiters);
extern char  *get_token_tq(char *source, char *token_start,
                           char *token_end, char *quote_start, char *quote_end);

/*   -- String routines: */
extern char *trim_spaces(char *s);
extern char *replace_chars(char *string, char *from, char *to);
extern char *rcdelete(char *string, char c_lower, char c_upper);
extern char *compress(char *string, char *chars_to_compress);
extern char *delete_chars(char *s, char *t);
extern char *delete_bounding(char *string, char *chars_to_delete);
extern char *str_toupper(char *string);
extern char *str_tolower(char *string);
extern long is_blank(char *string);
extern char *str_in(char *string, char *sub_string);
extern char *str_inn(char *string, char *sub_string, long n_char_to_check);
extern char *insert(char *place_to_insert, char *string_to_insert);
extern char *pad_with_spaces(char *s, int n_spaces);
extern char *cp_str(char **target, char *source);
extern char *cpn_str(char **target, char *source, long n_characters);
extern long edit_string(char *text, char *edit);
extern void edit_strings(char **string, long strings, char *buffer, char *edit);


long binaryInsert(void **array, long members, void *newMember, 
             int (*compare)(void *c1, void *c2), long *duplicate);
long binaryIndexSearch(void **array, long members, void *key, 
                       int (*compare)(void *c1, void *c2));

/* sort routines (previously sort.h) */
#if !defined(_MDBSORT_INCLUDED_)
#define _MDBSORT_INCLUDED_ 1 

extern int double_cmpasc(const void *a, const void *b);
extern int double_cmpdes(const void *a, const void *b);
extern void double_copy(void *a, void *b);
extern int float_cmpasc(const void *a, const void *b);
extern int float_cmpdes(const void *a, const void *b);
extern void float_copy(void *a, void *b);
extern int long_cmpasc(const void *a, const void *b);
extern int long_cmpdes(const void *a, const void *b);
extern void long_copy(void *a, void *b);
extern int string_cmpasc(const void *a, const void *b);
extern int string_cmpdes(const void *a, const void *b);
extern void string_copy(void *a, void *b);
extern int row_compare(const void *a, const void *b);
extern void row_copy(void *a, void *b);
extern void set_up_row_sort(int sort_by_column, size_t n_columns,
    size_t element_size,
    int (*compare)(const void *a, const void *b));
extern int unique(void *base, size_t n_items, size_t size,
    int (*compare)(const void *a, const void *b),
    void (*copy)(void *a, void *b));
#endif

/* string array matching (previously match_string.h): */
#if !defined(_MATCH_STRING_)
extern long match_string(char *string, char **option_list, long n_options,
                        long match_mode_flags);

#define _MATCH_STRING_ 1

#define DCL_STYLE_MATCH 0
#define UNIQUE_MATCH DCL_STYLE_MATCH
#define CASE_SENSITIVE 1
#define MATCH_WHOLE_STRING 2
#define RETURN_FIRST_MATCH 8
#define EXACT_MATCH (CASE_SENSITIVE|MATCH_WHOLE_STRING|RETURN_FIRST_MATCH)

#endif

extern char *clean_filename(char *filename);
extern long fexists(char *filename);
extern char *exp_notation(double x, long n1, long n2);
extern void add_to_headers(char **header, long n_headers, char **item,
    long min_width, long format_index);
long replaceFile(char *file, char *replacement);
long replaceFileAndBackUp(char *file, char *replacement);
void add_to_standard_headers(char *name_header, char *unit_header,
    char *printf_string, char *new_name, char *new_unit, char *new_format,
    long min_width);
extern long format_length(char *format_specifier);
extern char *sbinary(char *s, int len, long n);
extern long bitsSet(unsigned long data);
extern void interpret_escapes(char *s);
extern int replace_string(char *target, char *source, char *orig, char *new);

extern char **wild_list(int *n_names_ret, int **origin_ret, char **item_list,
        int num_items);
int wild_match(char *string, char *template);
int wild_match_ci(char *string, char *template);
char *strchr_ci(char *s, char c);
int strcmp_ci(const char *s, const char *t);
char *expand_ranges(char *template);
int has_wildcards(char *template);
char *unescape_wildcards(char *template);

/*   -- Routines for flagging and aborting on errors: */
extern void bomb(char *error_message, char *usage_message);
extern long bombre(char *error_message, char *usage_message, long return_value);
extern long err_mess(long status, char *routine_name, char *message);
extern long err_mess_sys(long status, char *routine_name, char *message);
extern void fatal_err(long error_code, char *error_message);

/*   -- IO routines: */
extern char *ffgets(char *target, long target_length, FILE *file_pointer);
extern FILE *fopen_e(char *file_name, char *open_mode, long error_mode);
#define FOPEN_EXIT_ON_ERROR 0
#define FOPEN_RETURN_ON_ERROR 1
#define FOPEN_INFORM_OF_OPEN  2
#define FOPEN_SAVE_IF_EXISTS  4
extern void backspace(long n);

/* terminal IO routines: */
extern int tt_attach(void);
extern char rawread(void);
extern void tt_detach(void);

/*   -- Miscellaneous routines: */
extern long log_usage(char *log_file_name, char *program_name);
extern char *tmpname(char *target);
char *mtime(void);


/********************* end of mdblib routines *****************************/

/************************ mathematical stuff *******************************/
#ifndef _MDB_MTH_
#define _MDB_MTH_ 1

#include <math.h>
#include <float.h>
#include <limits.h>

#define FABS(x) fabs(x)
#define SIGN(x) ((x)<0?-1:((x)>0?1:0))
#define IS_NEGATIVE(x) (SIGN(x)==-1)
#define IS_POSITIVE(x) (SIGN(x)==1)

/* mdbmth routines */
extern int fixcount(char *filename, long n_points);
extern long factorial(long n);
extern double dfactorial(long n);
extern double g_int(double (*function)(), double lower_limit,
                    double upper_limit, long n_panels, double error_limit);
extern double simpson(double (*function)(), double lower_limit,
                    double upper_limit, long n_panels);
void trapint(double *x, double *y, long n, double *integral);
void trapint_ep(double *x, double *y, double *sx, double *sy, long n, double *integral, double *error);
extern double ipow(double base, long power);
extern double amod(double x, double m);
extern double zeroInterp(double (*function)(), double x_initial, double x_final,
                     double x_step, double effective_zero);
extern double zeroIntHalve(double (*function)(), double x_initial, double x_final,
                     double x_step, double effective_zero);
extern double zeroNewton(double (*function)(), double x_initial, double dx_deriv,
                     long n_passes, double effective_zero);
extern double poly(double *coef, long n_coefs, double x);
extern double dpoly(double *coef, long n_coefs, double x);
extern double polyp(double *coef, long *power, long n_coefs, double x);
extern double dpolyp(double *coef, long *power, long n_coefs, double x);
extern double K_cei(double k);
extern double E_cei(double k);
extern double dK_cei(double k);
extern double dE_cei(double k);
extern double celliptic(double kc, double p, double a, double b);
extern float drand(long dummy);
extern double rdrand(double lower_limit, double upper_limit);
extern void r_theta_rand(double *r, double *theta, double r_min,
                           double r_max);
extern double random_1(long iseed);
extern double random_2(long iseed);
extern double random_3(long iseed);
extern double urandom_gauss(long iseed);
extern double gauss_rn(long iseed, double (*urandom)(long iseed1));
extern double gauss_rn_lim(double mean, double sigma, double limit_in_sigmas, double (*urandom)(long iseed));
extern double KS_Qfunction(double lambda);
extern double twoVariableKStest(double *d1, long n1, double *d2, long n2, double *MaxCDFerror);
extern double linearCorrelationCoefficient(double *data1, double *data2, long rows, long *count);
extern double linearCorrelationSignificance(double r, long rows);
extern double betaInc(double x, double a, double b);
extern double betaComp(double a, double b);
extern double gammaP(double a, double x);
extern double gammaQ(double a, double x);
extern double tTailSigLevel(double t0, long nu, long tails);
extern double FSigLevel(double var1, double var2, long nu1, long nu2);
extern double rSigLevel(double r0, long nu);
double ChiSqrSigLevel(double ChiSquared0, long nu);

extern long complex_sp_fft(float *data, long nn, long isign);
extern long complex_dp_fft(double *data, long nn, long isign);
extern long real_sp_fft(float *data, long n, long isign);
extern long real_dp_fft(double *data, long n, long isign);
extern void sp_digital_filter(float *t, float *y, long n_pts, 
    float **t_out, float **y_out, long *n_out,
    long do_window, double high_pass_freq1, double high_pass_freq2,
    double low_pass_freq1, double low_pass_freq2, long do_pad,
    long block_dc, long zp_spectrum, double threshold,
    long filter_points, double **filter);                              
extern void dp_digital_filter(double *t, double *y, long n_pts,
    double **t_out, double **y_out, long *n_out, long do_window,
    double high_pass_freq1, double high_pass_freq2,
    double low_pass_freq1, double low_pass_freq2, long do_pad,
    long do_truncate, long block_dc, long zp_spectrum, double threshold,
    long filter_points, double **filter);
void FFTderivative(double *T, double *Y, long n_pts0,
    double **T_out, double **Y_out, long *n_out, long do_pad,
    long do_truncate, long zp_spectrum);
extern long power_of_2(long n);
extern long is_prime(long number);
extern long smallest_factor(long number);
extern long next_prime_factor(long *number);
long largest_prime_factor(long number);
extern void copy_sp_array(float  **c, float  *o, long n);
extern void copy_dp_array(double **c, double *o, long n);
extern double bessel_Jn(double z, long n);
extern double bessel_Yn(double z, long n);
extern double Ai(double x), Bi(double x), Aip(double x),
        Bip(double x);

long lsfn(double *xd, double *yd, double *sy,
    long nd, long nf, double *coef, double *s_coef,
    double *chi, double *diff);
long lsfp(double *xd, double *yd, double *sy,
    long n_pts, long n_terms, long *power, double *coef, double *s_coef,
    double *chi, double *diff);
long lsfg(double *xd, double *yd, double *sy,
    long n_pts, long n_terms, long *order,
    double *coef, double *s_coef, double *chi, double *diff,
    double (*fn)(double x, long ord));


#define DIFFEQ_ZERO_STEPSIZE -3
#define DIFFEQ_CANT_TAKE_STEP -2
#define DIFFEQ_OUTSIDE_INTERVAL -1
#define DIFFEQ_XI_GT_XF 0
#define DIFFEQ_SOLVED 1
#define DIFFEQ_SOLVED_ALREADY 1
#define DIFFEQ_ZERO_FOUND 2
#define DIFFEQ_END_OF_INTERVAL 3
char *diffeq_result_description(long return_code);

extern long rk_odeint(
    double *y_i, void (*derivs)(double *yp, double *y, double x),
    long n_eq, double *accuracy, long *accmode, double *tiny, long *misses,
    double *x0, double xf, double x_accuracy, double h_start, double h_max,
    double *h_rec, double (*exfn)(double *yp, double *y, double x),
    double exit_accuracy, long n_to_skip,
    void (*store_data)(double *yp, double *y, double x, double exval)  );
extern long rk_odeint1(
    double *y_i, void (*derivs)(double *yp, double *y, double x),
    long n_eq, double *accuracy, long *accmode,
    double *tiny, long *misses, double *x0, double xf, double x_accuracy,
    double h_start, double h_max, double *h_rec );
extern long rk_odeint2(
    double *y_i, void (*derivs)(double *qp, double *q, double t), long n_eq,
    double *accuracy, long *accmode,
    double *tiny, long *misses, double *x0, double xf, double x_accuracy,
    double h_start, double h_max, double *h_rec, double exit_value,
    long i_exit_value, double exit_accuracy, long n_to_skip );
extern long rk_odeint3(
    double *y_i, void (*derivs)(double *yp, double *y, double x), long n_eq,
    double *accuracy, long *accmode, double *tiny, long *misses,
    double *x0, double xf, double x_accuracy, double h_start, double h_max,
    double *h_rec, double (*exfn)(double *yp, double *y, double x),
    double exit_accuracy);
extern long rk_odeint4(
    double *y_i, void (*derivs)(double *qp, double *q, double t), long n_eq,
    double *accuracy, long *accmode,
    double *tiny, long *misses, double *x0, double xf, double x_accuracy,
    double h_start, double h_max, double *h_rec, double exit_value,
    long i_exit_value, double exit_accuracy, long n_to_skip,
    void (*store_data)(double *qp, double *q, double t, double exfn) );
extern long rk_odeint_na(
    double *y_i, void (*derivs)(double *yp, double *y, double x),
    long n_eq, double *null1, long *null2, double *null3, long *null4,
    double *x0, double xf, double dummy1,
    double h, double dummy2, double *dummy3 );
extern long rk_odeint3_na(
    double *y_i, void (*derivs)(double *yp, double *y, double x), long n_eq,
    double *accuracy, long *accmode, double *tiny, long *misses,
    double *x0, double xf, double x_accuracy, double h_start, double h_max,
    double *h_rec, double (*exfn)(double *yp, double *y, double x),
    double exit_accuracy);
extern long bs_odeint(
    double *y_i, void (*derivs)(double *yp, double *y, double x), long n_eq,
    double *accuracy, long *accmode, double *tiny, long *misses,
    double *x0, double xf, double x_accuracy, double h_start, double h_max,
    double *h_rec,
    double (*exfn)(double *yp, double *y, double x), double exit_accuracy, long n_to_skip,
    void (*store_data)(double *qp, double *q, double t, double exfn_value)  );
extern long bs_odeint1(
    double *y_i, void (*derivs)(double *yp, double *y, double x), long n_eq,
    double *accuracy, long *accmode, double *tiny, long *misses,
    double *x0, double xf, double x_accuracy, double h_start, double h_max,
    double *h_rec );
extern long bs_odeint2(
    double *y_i, void (*derivs)(double *qp, double *q, double t), long n_eq,
    double *accuracy, long *accmode, double *tiny, long *misses,
    double *x0, double xf, double x_accuracy, double h_start, double h_max,
    double *h_rec,
    double exit_value, long i_exit_value, double exit_accuracy, long n_to_skip );
extern long bs_odeint3(
    double *y_i, void (*derivs)(double *yp, double *y, double x), long n_eq,
    double *accuracy, long *accmode, double *tiny, long *misses,
    double *x0, double xf, double x_accuracy, double h_start, double h_max,
    double *h_rec, double (*exfn)(double *yp, double *y, double x),
    double exit_accuracy);
extern long bs_odeint4(
    double *y_i, void (*derivs)(double *qp, double *q, double t), long n_eq,
    double *accuracy, long *accmode,
    double *tiny, long *misses, double *x0, double xf, double x_accuracy,
    double h_start, double h_max, double *h_rec, double exit_value,
    long i_exit_value, double exit_accuracy, long n_to_skip,
    void (*store_data)(double *qp, double *q, double t, double exfn) );
extern long bss_odeint(
    float *y_i, void (*derivs)(float *yp, float *y, float x), long n_eq,
    float *accuracy, long *accmode, float *tiny, long *misses,
    float *x0, float xf, float x_accuracy, float h_start, float h_max,
    float *h_rec, float (*exfn)(float *yp, float *y, float x),
    float exit_accuracy, long n_to_skip,
    void (*store_data)(float *yp, float *y, float x, float exval)  );
extern long bss_odeint1(
    float *y_i, void (*derivs)(float *yp, float *y, float x), long n_eq,
    float *accuracy, long *accmode, float *tiny, long *misses, float *x0,
    float xf, float x_accuracy, float h_start, float h_max, float *h_rec );
extern long bss_odeint3(
    float *y_i, void (*derivs)(float *yp, float *y, float x), long n_eq,
    float *accuracy, long *accmode, float *tiny, long *misses,
    float *x0, float xf, float x_accuracy, float h_start, float h_max,
    float *h_rec, float (*exfn)(float *yp, float *y, float x),
    float exit_accuracy);
extern long rks_odeint(
    float *y_i, void (*derivs)(float *dydx, float *y, float x),
    long n_eq, float *accuracy, long *accmode, float *tiny, long *misses,
    float *x0, float xf, float x_accuracy, float h_start, float h_max,
    float *h_rec, float (*exit_func)(float *dydx, float *y, float x),
    float exit_accuracy, long n_to_skip,
    void (*store_data)(float *dydx, float *y, float x, float exval)  );
extern long rks_odeint1(
    float *y_i, void (*derivs)(float *yp, float *y, float x), long n_eq,
    float *accuracy, long *accmode, float *tiny, long *misses, float *x0,
    float xf, float x_accuracy, float h_start, float h_max, float *h_rec);
extern long rks_odeint2(
    float *y_i, void (*derivs)(float *yp, float *y, float x), long n_eq,
    float *accuracy, long *accmode, float *tiny, long *misses, float *x0,
    float xf, float x_accuracy, float h_start, float h_max, float *h_rec,
    float exit_value, long i_exit_value, float exit_accuracy, long n_to_skip);
extern long rks_odeint_na(
    float *y_i, void (*derivs)(float *yp, float *y, float x),
    long n_eq, float *null1, long *null2, float *null3, long *null4,
    float *x0, float xf, float dummy1,
    float h, float dummy2, float *dummy3 );
extern long stiff_odeint1(
    double *y_i, void (*derivs)(double *yp, double *y, double x),
    long n_eq, double *accuracy, long *accmode,
    double *tiny, long *misses, double *x0, double xf, double x_accuracy,
    double h_start, double h_max, double *h_rec );
void mmid(double *y, double *dydx, long nvar, double xs, double htot,
    long nstep, double *yout, void (*derivs)(double *dydxa, double *ya, double xa)
    );
void mmid2(double *y, double *dydx, long nvar, double xs, double htot,
    long nstep, double *yout, void (*derivs)(double *dydxa, double *ya, double xa)
    );

extern double minc(double (*fn)(double *param), double *x, double *dx,
    double *dx_lim, double *xlo, double *xhi, long np, long ns_max,
    long p_flag);

long simplexMin(double *yReturn, double *xGuess, double *dxGuess, double *xLowerLimit,
    double *xUpperLimit, long dimensions, double target, double tolerance, double (*func)(double *x, long *invalid), 
    void (*report)(double ymin, double *xmin, long pass, long n_evals, long n_dim),
    long maxEvaluations, long maxPasses);
long simplexMinimization(double **simplexVector, double *fValue, double *coordLowerLimit,
    double *coordUpperLimit, long dimensions, double target, double tolerance, long tolerance_mode,
    double (*function)(double *x, long *invalid), long maxEvaluations, long *evaluations);
long grid_search_min(double *best_result, double *best_x, double *lower, double *upper,
    double *step, long n_dimen, double (*func)(double *x, long *invalid));
long grid_sample_min(double *best_result, double *best_x, double *lower, double *upper,
    double *step, long n_dimen, double (*func)(double *x, long *invalid), double sample_fraction);

long advance_values(double *value, long *value_index, double *initial, double *step, long n_values,
    long *counter, long *max_count, long n_indices);
long advance_counter(long *counter, long *max_count, long n_indices);

extern long compute_average(double *value, double *x, long n);
extern long compute_middle(double *value, double *x, long n);
extern long compute_median(double *value, double *x, long n);
extern long compute_percentile(double *value, double *x, long n, double percentile);

extern long find_average(double *value, double *x, long n);
extern long find_middle(double *value, double *x, long n);
extern long find_median(double *value, double *x, long n);
extern long find_percentile(double *value, double *x, long n, double percentile);
extern long find_median_of_row(double *value, double **x, long index, long n);

extern long make_histogram(double *hist, long n_bins, double lo, double hi, double *data,
    long n_pts, long new_start);
extern long make_histogram_weighted(double *hist, long n_bins, double lo, double hi, double *data,
    long n_pts, long new_start, double *weight);

long findCrossingPoint(long start, double *data, long points, double level, long direction,
                       double *interpData, double *result);
long findTopBaseLevels(double *top, double *base, double *data, long points,
                       long bins, double sigmasRequired);

extern double standardDeviation(double *x, long n);
extern double rmsValue(double *y, long n);
extern double arithmeticAverage(double *y, long n);
extern double meanAbsoluteDeviation(double *y, long n);
extern long computeMoments(double *mean, double *rms, double *standardDev,
          double *meanAbsoluteDev, double *x, long n);
extern long computeCorrelations(double *C11, double *C12, double *C22, double *x, double *y, long n);
extern long computeWeightedMoments(double *mean, double *rms, double *standardDev,
          double *meanAbsoluteDev, double *x, double *w, long n);
extern long accumulateMoments(double *mean, double *rms, double *standardDev,
          double *x, long n, long reset);
extern long accumulateWeightedMoments(double *mean, double *rms, double *standardDev,
          double *x, double *w, long n, long reset);
extern double weightedAverage(double *y, double *w, long n);
extern double weightedRMS(double *y, double *w, long n);
extern double weightedMAD(double *y, double *w, long n);
extern double weightedStDev(double *y, double *w, long n);

extern int find_min_max(double *min, double *max, double *list, long n);
extern int index_min_max(long *imin, long *imax, double *list, long n);
extern int assign_min_max(double *min, double *max, double val);
extern int find_min_max_2d(double *min, double *max, double **value,
            long n1, long n2);
extern int find_min_max_2d_float(float *min, float *max, float **value,
    long n1, long n2);
extern int find_min(double *min, double *loc, double *c1, double *c2, long n);
extern int find_max(double *max, double *loc, double *c1, double *c2, long n);
extern double max_in_array(double *array, long n);
extern double min_in_array(double *array, long n);
extern double interp(double *y, double *x, long n, double x0, long warn, long order, long *returnCode);
int interpolate_minimum(double *fmin, double *zmin, double *value, double z_lo,
    double z_hi, long n);
double LagrangeInterp(double *x, double *f, long order, double x0, long *returnCode);

#define iceil(x) ((int)ceil(x))
#define round(x) ((int)((x)+.5))

#define MIN(x,y) ( ((x)>(y)) ? (y) : (x))
#define MAX(x,y) ( ((x)<(y)) ? (y) : (x))

#define SWAP_LONG(x, y) {long tmp_swap_long; tmp_swap_long=(x); (x)=(y); (y)=tmp_swap_long; }
#define SWAP_INT(x, y) {int tmp_swap_int; tmp_swap_int=(x); (x)=(y); (y)=tmp_swap_int; }
#define SWAP_SHORT(x, y) {short tmp_swap_short; tmp_swap_short=(x); (x)=(y); (y)=tmp_swap_short; }
#define SWAP_DOUBLE(x, y) {double tmp_swap_double; tmp_swap_double=(x); (x)=(y); (y)=tmp_swap_double; }
#define SWAP_FLOAT(x, y) {float tmp_swap_float; tmp_swap_float=(x); (x)=(y); (y)=tmp_swap_float; }
#define SWAP_PTR(x, y) {void *tmp_swap_ptr; tmp_swap_ptr=(x); (x)=(y); (y)=tmp_swap_ptr; }

#define INTERPOLATE(y1, y2, x1, x2, x0) (((y2)-(y1))/((x2)-(x1))*((x0)-(x1)) + (y1))

#define sqr(x)  ipow(x, 2)
#define pow2(x) sqr(x)
#define pow3(x) ipow(x, 3)
#define pow4(x) ipow(x, 4)
#define pow5(x) ipow(x, 5)

#include "constants.h"

#endif  /* _MDB_MTH_ */

/* machine-specific include file: */
#ifdef VAX_VMS
#include "mdbvax.h"
#endif
#ifdef SUNOS4
#include "mdbsunos4.h"
#endif
#ifdef __TURBOC__
#include "mdbtc.h"
#endif

#endif /* _MDB_ */

